"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = __importDefault(require("bluebird"));
const chalk_1 = __importDefault(require("chalk"));
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const utils = __importStar(require("../../server-utils"));
const event_source_1 = __importDefault(require("../event-source"));
const report_subscriber_1 = __importDefault(require("./report-subscriber"));
const runner_1 = __importDefault(require("./runner"));
const utils_1 = require("./utils");
const common_utils_1 = require("../../common-utils");
const report_builder_1 = __importDefault(require("../../report-builder/report-builder"));
const reporterHelper = __importStar(require("../../reporter-helpers"));
class HermioneRunner {
    constructor(paths, hermione, { program, pluginConfig, options }) {
        this.testFiles = [].concat(paths);
        this.hermione = hermione;
        this.treeInternal = null;
        this.collection = null;
        this.globalOpts = program;
        this.guiOpts = options;
        this.reportPath = pluginConfig.path;
        this.eventSource = new event_source_1.default();
        this.reportBuilder = new report_builder_1.default(hermione, pluginConfig);
        this.tests = {};
    }
    get config() {
        return this.hermione.config;
    }
    get tree() {
        return this.treeInternal;
    }
    initialize() {
        return this.readTests().then((collection) => {
            this.collection = collection;
            this.handleRunnableCollection();
            this.subscribeOnEvents();
        });
    }
    run(tests = []) {
        const { grep, set: sets, browser: browsers } = this.globalOpts;
        const formattedTests = lodash_1.default.flatMap([].concat(tests), test => utils_1.formatTests(test));
        return runner_1.default(this.collection, formattedTests).run((collection) => this.hermione.run(collection, { grep, sets, browsers }));
    }
    fillTestsTree() {
        const { autoRun } = this.guiOpts;
        this.treeInternal = Object.assign(this.reportBuilder.getResult(), { autoRun, gui: true });
        this.treeInternal = Object.assign({}, this.applyReuseData(this.treeInternal));
        this.reportBuilder.setStats({
            failed: this.treeInternal.failed,
            passed: this.treeInternal.passed,
            retries: this.treeInternal.retries,
            skipped: this.treeInternal.skipped,
            total: this.treeInternal.total,
        });
    }
    updateReferenceImage(tests) {
        const reportBuilder = this.reportBuilder;
        return bluebird_1.default.map(tests, (test) => {
            const updateResult = this.prepareUpdateResult(test);
            const formattedResult = reportBuilder.format(updateResult);
            return bluebird_1.default.map(updateResult.imagesInfo, (imageInfo) => {
                const { stateName } = imageInfo;
                return reporterHelper
                    .updateReferenceImage(formattedResult, this.reportPath, stateName)
                    .then(() => {
                    this.hermione.emit(this.hermione.events.UPDATE_RESULT, lodash_1.default.extend(updateResult, { imagePath: imageInfo.imagePath }));
                });
            }).then(() => {
                reportBuilder.addUpdated(updateResult);
                return utils_1.findTestResult(reportBuilder.getSuites(), formattedResult.prepareTestResult());
            });
        });
    }
    finalize() {
        this.reportBuilder.saveDataFileSync();
    }
    addClient(connection) {
        this.eventSource.addConnection(connection);
    }
    sendClientEvent(event, data) {
        this.eventSource.emit(event, data);
    }
    handleRunnableCollection() {
        this.collection.eachTest((test, browserId) => {
            if (test.disabled || test.silentSkip) {
                return;
            }
            const testId = utils_1.formatId(test.id(), browserId);
            this.tests[testId] = lodash_1.default.extend(test, { browserId });
            test.pending
                ? this.reportBuilder.addSkipped(test)
                : this.reportBuilder.addIdle(test);
        });
        this.fillTestsTree();
    }
    subscribeOnEvents() {
        report_subscriber_1.default(this.hermione, this.reportBuilder, this.eventSource, this.reportPath);
    }
    prepareUpdateResult(test) {
        const { browserId, attempt } = test;
        const fullTitle = utils_1.mkFullTitle(test);
        const testId = utils_1.formatId(utils_1.getShortMD5(fullTitle), browserId);
        const testResult = this.tests[testId];
        const { sessionId, url } = test.metaInfo;
        const imagesInfo = test.imagesInfo.map((imageInfo) => {
            const { stateName } = imageInfo;
            const imagePath = this.hermione.config.browsers[browserId]
                .getScreenshotPath(testResult, stateName);
            return lodash_1.default.extend(imageInfo, { imagePath });
        });
        return lodash_1.default.merge({}, testResult, {
            attempt,
            imagesInfo,
            sessionId,
            meta: { url },
            updated: true,
        });
    }
    readTests() {
        const { grep, set: sets, browser: browsers } = this.globalOpts;
        return this.hermione.readTests(this.testFiles, { grep, sets, browsers });
    }
    applyReuseData(treeInternal) {
        if (!treeInternal.suites) {
            return;
        }
        const reuseData = this.loadReuseData();
        if (!lodash_1.default.isEmpty(reuseData.suites)) {
            treeInternal.suites.map((suite) => applyReuse(reuseData)(suite));
        }
        if (reuseData.failed) {
            treeInternal.failed = parseInt(reuseData.failed, 10);
        }
        if (reuseData.passed) {
            treeInternal.passed = parseInt(reuseData.passed, 10);
        }
        if (reuseData.retries) {
            treeInternal.retries = parseInt(reuseData.retries, 10);
        }
        if (reuseData.skipped) {
            treeInternal.skipped = parseInt(reuseData.skipped, 10);
        }
        if (reuseData.total) {
            treeInternal.total = parseInt(reuseData.total, 10);
        }
        return treeInternal;
    }
    loadReuseData() {
        try {
            return require(path_1.default.resolve(this.reportPath, 'data'));
        }
        catch (e) {
            utils.logger.warn(chalk_1.default.yellow(`Nothing to reuse in ${this.reportPath}`));
            return {};
        }
    }
}
exports.default = HermioneRunner;
function applyReuse(reuseData) {
    let isBrowserResultReused = false;
    const reuseBrowserResult = (suite) => {
        if (suite.children) {
            suite.children = suite.children.map(reuseBrowserResult);
            return isBrowserResultReused
                ? lodash_1.default.set(suite, 'status', getReuseStatus(reuseData.suites, suite))
                : suite;
        }
        return lodash_1.default.set(suite, 'browsers', suite.browsers.map((bro) => {
            const browserResult = getReuseBrowserResult(reuseData.suites, suite.suitePath, bro.name);
            if (browserResult) {
                isBrowserResultReused = true;
                suite.status = getReuseStatus(reuseData.suites, suite);
            }
            return lodash_1.default.extend(bro, browserResult);
        }));
    };
    return reuseBrowserResult;
}
function getReuseStatus(reuseSuites, { suitePath, status: defaultStatus }) {
    const reuseNode = common_utils_1.findNode(reuseSuites, suitePath);
    return lodash_1.default.get(reuseNode, 'status', defaultStatus);
}
function getReuseBrowserResult(reuseSuites, suitePath, browserId) {
    const reuseNode = common_utils_1.findNode(reuseSuites, suitePath);
    return lodash_1.default.find(lodash_1.default.get(reuseNode, 'browsers'), { name: browserId });
}
