"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const lodash_1 = __importDefault(require("lodash"));
const bluebird_1 = __importDefault(require("bluebird"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const common_utils_1 = require("../common-utils");
const utils_1 = require("./utils");
const fs = bluebird_1.default.promisifyAll(fs_extra_1.default);
class DataTree {
    static create(initialData, destPath) {
        return new this(initialData, destPath);
    }
    constructor(initialData, destPath) {
        this.data = initialData;
        this.destPath = destPath;
    }
    async mergeWith(dataCollection) {
        // make it serially in order to perform correct merge/permutation of images and datas
        await bluebird_1.default.each(lodash_1.default.toPairs(dataCollection), async ([srcPath, data]) => {
            this.srcPath = srcPath;
            this.mergeSkips(data.skips);
            await this.mergeSuites(data.suites);
        });
        return this.data;
    }
    mergeSkips(srcSkips) {
        srcSkips.forEach((skip) => {
            if (!lodash_1.default.find(this.data.skips, { suite: skip.suite, browser: skip.browser })) {
                this.data.skips.push(skip);
            }
        });
    }
    async mergeSuites(srcSuites) {
        await bluebird_1.default.map(srcSuites, async (suite) => {
            await this.mergeSuiteResult(suite);
        });
    }
    async mergeSuiteResult(suite) {
        const existentSuite = common_utils_1.findNode(this.data.suites, suite.suitePath);
        if (!existentSuite) {
            await this.addSuiteResult(suite);
            return;
        }
        if (suite.children) {
            await bluebird_1.default.map(suite.children, (childSuite) => this.mergeSuiteResult(childSuite));
        }
        else {
            await this.mergeBrowserResult(suite);
        }
    }
    async mergeBrowserResult(suite) {
        await bluebird_1.default.map(suite.browsers, async (bro) => {
            const existentBro = this.findBrowserResult(suite.suitePath, bro.name);
            if (!existentBro) {
                await this.addBrowserResult(bro, suite.suitePath);
                return;
            }
            this.moveTestResultToRetries(existentBro);
            await this.addTestRetries(existentBro, bro.retries);
            await this.changeTestResult(existentBro, bro.result, suite.suitePath);
        });
    }
    async addSuiteResult(suite) {
        if (suite.suitePath.length === 1) {
            this.data.suites.push(suite);
        }
        else {
            const existentParentSuite = common_utils_1.findNode(this.data.suites, suite.suitePath.slice(0, -1));
            existentParentSuite.children.push(suite);
        }
        this.mergeStatistics(suite);
        await this.moveImages(suite, { fromFields: ['result', 'retries'] });
    }
    async addBrowserResult(bro, suitePath) {
        const existentParentSuite = common_utils_1.findNode(this.data.suites, suitePath);
        existentParentSuite.browsers.push(bro);
        this.mergeStatistics(bro);
        await this.moveImages(bro, { fromFields: ['result', 'retries'] });
    }
    moveTestResultToRetries(existentBro) {
        existentBro.retries.push(existentBro.result);
        this.data.retries += 1;
        const statName = utils_1.getStatNameForStatus(existentBro.result.status);
        this.data[statName] -= 1;
    }
    async addTestRetries(existentBro, retries) {
        await bluebird_1.default.mapSeries(retries, (retry) => this.addTestRetry(existentBro, retry));
    }
    async addTestRetry(existentBro, retry) {
        const newAttempt = existentBro.retries.length;
        await this.moveImages(retry, { newAttempt });
        const attemptedRetry = this.changeFieldsWithAttempt(retry, { newAttempt });
        existentBro.retries.push(attemptedRetry);
        this.data.retries += 1;
    }
    async changeTestResult(existentBro, result, suitePath) {
        await this.moveImages(result, { newAttempt: existentBro.retries.length });
        existentBro.result = this.changeFieldsWithAttempt(result, { newAttempt: existentBro.retries.length });
        const statName = utils_1.getStatNameForStatus(existentBro.result.status);
        this.data[statName] += 1;
        if (!common_utils_1.isSkippedStatus(existentBro.result.status)) {
            common_utils_1.setStatusForBranch(this.data.suites, suitePath, existentBro.result.status);
        }
    }
    mergeStatistics(node) {
        const testResultStatuses = utils_1.getDataFrom(node, { fieldName: 'status', fromFields: 'result' });
        testResultStatuses.forEach((testStatus) => {
            const statName = utils_1.getStatNameForStatus(testStatus);
            if (this.data.hasOwnProperty(statName)) {
                this.data.total += 1;
                this.data[statName] += 1;
            }
        });
        const testRetryStatuses = utils_1.getDataFrom(node, { fieldName: 'status', fromFields: 'retries' });
        this.data.retries += testRetryStatuses.length;
    }
    async moveImages(node, { newAttempt, fromFields }) {
        await bluebird_1.default.map(utils_1.getImagePaths(node, fromFields), async (imgPath) => {
            const srcImgPath = path_1.default.resolve(this.srcPath, imgPath);
            const destImgPath = path_1.default.resolve(this.destPath, lodash_1.default.isNumber(newAttempt) ? imgPath.replace(/\d+(?=.png$)/, newAttempt) : imgPath);
            await fs.move(srcImgPath, destImgPath, { overwrite: true });
        });
    }
    changeFieldsWithAttempt(testResult, { newAttempt }) {
        const pathes = ['expectedPath', 'actualPath', 'diffPath'];
        const imagesInfo = testResult.imagesInfo.map((imageInfo) => {
            return lodash_1.default.mapValues(imageInfo, (val, key) => {
                return pathes.includes(key)
                    ? val.replace(/\d+(?=.png)/, newAttempt)
                    : val;
            });
        });
        return lodash_1.default.extend({}, testResult, { imagesInfo, attempt: newAttempt });
    }
    findBrowserResult(suitePath, browserId) {
        const existentNode = common_utils_1.findNode(this.data.suites, suitePath);
        return lodash_1.default.find(lodash_1.default.get(existentNode, 'browsers'), { name: browserId });
    }
}
exports.default = DataTree;
