"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = __importDefault(require("bluebird"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const find_package_json_1 = __importDefault(require("find-package-json"));
const git_url_parse_1 = __importDefault(require("git-url-parse"));
const child_process_1 = require("child_process");
const test_statuses_1 = require("../constants/test-statuses");
const server_utils_1 = require("../server-utils");
const common_utils_1 = require("../common-utils");
const test_result_1 = __importDefault(require("../test-result/test-result"));
const NO_STATE = 'NO_STATE';
class ReportBuilder {
    constructor(hermione, pluginConfig) {
        this.tree = { name: 'root' };
        this.skips = [];
        this.hermione = hermione;
        this.pluginConfig = pluginConfig;
        this.stats = {
            failed: 0,
            passed: 0,
            retries: 0,
            skipped: 0,
            total: 0,
        };
    }
    format(result) {
        return new test_result_1.default(result, this.hermione);
    }
    addIdle(result) {
        return this.addTestResult(this.format(result), { status: test_statuses_1.IDLE });
    }
    addSkipped(result) {
        const formattedResult = this.format(result);
        const { suite: { skipComment: comment, fullName: suite }, browserId: browser, } = formattedResult;
        this.skips.push({ suite, browser, comment });
        return this.addTestResult(formattedResult, {
            reason: comment,
            status: test_statuses_1.SKIPPED,
        });
    }
    addSuccess(result) {
        return this.addSuccessResult(this.format(result), test_statuses_1.SUCCESS);
    }
    addFail(result) {
        return this.addFailResult(this.format(result));
    }
    addError(result) {
        return this.addErrorResult(this.format(result));
    }
    setStats(stats) {
        this.stats = stats;
        return this;
    }
    addRetry(result) {
        const formattedResult = this.format(result);
        const retry = formattedResult.hasDiff()
            ? this.addFailResult(formattedResult)
            : this.addErrorResult(formattedResult);
        return retry;
    }
    save() {
        return this.saveDataFileAsync()
            .then(() => this.copyToReportDir(['index.html', 'report.min.js', 'report.min.css']))
            .then(() => this)
            .catch((e) => server_utils_1.logger.warn(e.message || e));
    }
    saveDataFileAsync() {
        return fs_extra_1.default
            .mkdirs(this.pluginConfig.path)
            .then(() => this.saveDataFile(fs_extra_1.default.writeFile));
    }
    saveDataFileSync() {
        fs_extra_1.default.mkdirsSync(this.pluginConfig.path);
        this.saveDataFile(fs_extra_1.default.writeFileSync);
    }
    getSuites() {
        return this.tree.children;
    }
    addUpdated(result) {
        const formattedResult = this.format(result);
        formattedResult.imagesInfo = (result.imagesInfo || []).map((imageInfo) => {
            const { stateName } = imageInfo;
            return lodash_1.default.extend(imageInfo, server_utils_1.getPathsFor(test_statuses_1.UPDATED, formattedResult, stateName));
        });
        return this.addSuccessResult(formattedResult, test_statuses_1.UPDATED);
    }
    get reportPath() {
        return path_1.default.resolve(`${this.pluginConfig.path}/index.html`);
    }
    getResult() {
        const { defaultView, baseHost, scaleImages, lazyLoadOffset, } = this.pluginConfig;
        let { gitUrl } = this.pluginConfig;
        if (!gitUrl) {
            const f = find_package_json_1.default();
            const repo = f.next().value.repository;
            if (repo) {
                gitUrl = typeof repo === 'string' ? repo : repo.url;
            }
        }
        if (gitUrl !== '') {
            const currentBranch = child_process_1.execSync('git rev-parse --abbrev-ref HEAD');
            const onlyUrl = git_url_parse_1.default(gitUrl);
            const pathBranch = `/blob/${currentBranch}`;
            const url = `https://${onlyUrl.source}/${onlyUrl.owner}/${onlyUrl.name}${pathBranch}`;
            gitUrl = url.trim();
        }
        this.sortTree();
        return lodash_1.default.extend({
            config: { defaultView, baseHost, scaleImages, lazyLoadOffset, gitUrl },
            skips: lodash_1.default.uniq(this.skips),
            suites: this.tree.children,
        }, this.stats);
    }
    addSuccessResult(formattedResult, status) {
        return this.addTestResult(formattedResult, { status });
    }
    addFailResult(formattedResult) {
        return this.addTestResult(formattedResult, { status: test_statuses_1.FAIL });
    }
    addErrorResult(formattedResult) {
        return this.addTestResult(formattedResult, {
            reason: formattedResult.error,
            status: test_statuses_1.ERROR,
        });
    }
    createTestResult(result, props) {
        const { browserId, suite, sessionId, description, imagesInfo, scenario, screenshot, testBody, multipleTabs, } = result;
        const { baseHost } = this.pluginConfig;
        const suiteUrl = suite.getUrl({ baseHost });
        const metaInfo = lodash_1.default.merge(result.meta, {
            sessionId,
            file: suite.file,
            url: suite.fullUrl,
        });
        const testResult = {
            description,
            imagesInfo,
            metaInfo,
            multipleTabs,
            scenario,
            suiteUrl,
            testBody,
            name: browserId,
            screenshot: Boolean(screenshot),
        };
        return Object.assign({}, testResult, props);
    }
    addTestResult(formattedResult, props) {
        const testResult = this.createTestResult(formattedResult, lodash_1.default.extend(props, { attempt: 0 }));
        const { suite, browserId } = formattedResult;
        const suitePath = suite.path.concat(formattedResult.state ? formattedResult.state.name : NO_STATE);
        const node = findOrCreate(this.tree, suitePath);
        node.browsers = Array.isArray(node.browsers) ? node.browsers : [];
        const existing = lodash_1.default.findIndex(node.browsers, { name: browserId });
        if (existing === -1) {
            formattedResult.attempt = testResult.attempt;
            formattedResult.image = server_utils_1.hasImage(formattedResult);
            extendTestWithImagePaths(testResult, formattedResult);
            if (common_utils_1.hasNoRefImageErrors(formattedResult)) {
                testResult.status = test_statuses_1.FAIL;
            }
            node.browsers.push({ name: browserId, result: testResult, retries: [] });
            common_utils_1.setStatusForBranch(this.tree, node.suitePath, testResult.status);
            return formattedResult;
        }
        const stateInBrowser = node.browsers[existing];
        const previousResult = lodash_1.default.cloneDeep(stateInBrowser.result);
        const statuses = [test_statuses_1.SKIPPED, test_statuses_1.RUNNING, test_statuses_1.IDLE];
        if (!statuses.includes(previousResult.status)) {
            testResult.attempt =
                testResult.status === test_statuses_1.UPDATED
                    ? formattedResult.attempt
                    : previousResult.attempt + 1;
            if (testResult.status !== test_statuses_1.UPDATED) {
                stateInBrowser.retries.push(previousResult);
            }
        }
        formattedResult.attempt = testResult.attempt;
        formattedResult.image = server_utils_1.hasImage(formattedResult);
        const { imagesInfo, status: currentStatus } = stateInBrowser.result;
        stateInBrowser.result = extendTestWithImagePaths(testResult, formattedResult, imagesInfo);
        if (!common_utils_1.hasFails(stateInBrowser)) {
            stateInBrowser.result.status = test_statuses_1.SUCCESS;
        }
        else if (common_utils_1.hasNoRefImageErrors(stateInBrowser.result)) {
            stateInBrowser.result.status = test_statuses_1.FAIL;
        }
        else if (stateInBrowser.result.status === test_statuses_1.UPDATED) {
            stateInBrowser.result.status = currentStatus;
        }
        common_utils_1.setStatusForBranch(this.tree, node.suitePath, testResult.status);
        return formattedResult;
    }
    saveDataFile(saveFn) {
        return saveFn(path_1.default.join(this.pluginConfig.path, 'data.js'), server_utils_1.prepareCommonJSData(this.getResult()), 'utf8');
    }
    sortTree(node = this.tree) {
        if (node.children) {
            node.children = lodash_1.default.sortBy(node.children, 'name');
            node.children.forEach((n) => this.sortTree(n));
        }
    }
    copyToReportDir(files) {
        return bluebird_1.default.map(files, (fileName) => {
            const from = path_1.default.resolve(__dirname, '../static', fileName);
            const to = path_1.default.join(this.pluginConfig.path, fileName);
            return fs_extra_1.default.copy(from, to);
        });
    }
}
exports.default = ReportBuilder;
const findOrCreate = (node, statePath) => {
    if (statePath.length === 0) {
        return node;
    }
    node.children = Array.isArray(node.children) ? node.children : [];
    const pathPart = statePath.shift();
    node.suitePath = node.suitePath || [];
    if (pathPart === NO_STATE) {
        return node;
    }
    let child = lodash_1.default.find(node.children, { name: pathPart });
    if (!child) {
        child = {
            name: pathPart,
            suitePath: node.suitePath.concat(pathPart),
        };
        node.children.push(child);
    }
    return findOrCreate(child, statePath);
};
const extendTestWithImagePaths = (test, formattedResult, oldImagesInfo = []) => {
    const newImagesInfo = formattedResult.getImagesInfo();
    if (test.status !== test_statuses_1.UPDATED) {
        return lodash_1.default.set(test, 'imagesInfo', newImagesInfo);
    }
    if (oldImagesInfo.length) {
        test.imagesInfo = oldImagesInfo;
        newImagesInfo.forEach((imageInfo) => {
            const { stateName } = imageInfo;
            let index = lodash_1.default.findIndex(test.imagesInfo, { stateName });
            index = index >= 0
                ? index
                : lodash_1.default.findLastIndex(test.imagesInfo);
            test.imagesInfo[index] = imageInfo;
        });
    }
    return test;
};
