"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const lodash_1 = __importDefault(require("lodash"));
const path_1 = __importDefault(require("path"));
const test_statuses_1 = require("./constants/test-statuses");
exports.getReferencePath = (testResult, stateName) => createPath('ref', testResult, stateName);
exports.getCurrentPath = (testResult, stateName) => createPath('current', testResult, stateName);
exports.getDiffPath = (testResult, stateName) => createPath('diff', testResult, stateName);
exports.getReferenceAbsolutePath = (testResult, reportDir, stateName) => {
    const referenceImagePath = exports.getReferencePath(testResult, stateName);
    return path_1.default.resolve(reportDir, referenceImagePath);
};
exports.getCurrentAbsolutePath = (testResult, reportDir, stateName) => {
    const currentImagePath = exports.getCurrentPath(testResult, stateName);
    return path_1.default.resolve(reportDir, currentImagePath);
};
exports.getDiffAbsolutePath = (testResult, reportDir, stateName) => {
    const diffImagePath = exports.getDiffPath(testResult, stateName);
    return path_1.default.resolve(reportDir, diffImagePath);
};
/**
 * @param kind - одно из значений 'ref', 'current', 'diff'
 * @param {StateResult} result
 * @param stateName - имя стэйта для теста
 */
const createPath = (kind, result, stateName) => {
    const attempt = result.attempt || 0;
    const imageDir = lodash_1.default.compact(['images', result.imageDir, stateName]);
    const components = imageDir.concat(`${result.browserId}~${kind}_${attempt}.png`);
    return path_1.default.join.apply(null, components);
};
// создает директорию и копирует скрин
exports.copyImageAsync = (srcPath, destPath) => {
    return exports.makeDirFor(destPath)
        .then(() => fs_extra_1.default.copy(srcPath, destPath));
};
/**
 * @param {TestStateResult} result
 * @param {String} destPath
 * @returns {Promise}
 */
exports.saveDiff = (result, destPath) => {
    return exports.makeDirFor(destPath)
        .then(() => result.saveDiffTo(destPath));
};
/**
 * @param {String} destPath
 */
exports.makeDirFor = (destPath) => {
    return fs_extra_1.default.mkdirs(path_1.default.dirname(destPath));
};
exports.logger = lodash_1.default.pick(console, ['log', 'warn', 'error']);
exports.logPathToHtmlReport = (reportBuilder) => {
    const reportPath = `file://${reportBuilder.reportPath}`;
    exports.logger.log(`Your HTML report is here: ${chalk_1.default.yellow(reportPath)}`);
};
exports.logError = (e) => {
    exports.logger.error(e.stack);
};
exports.getPathsFor = (status, formattedResult, stateName) => {
    if (status === test_statuses_1.SUCCESS || status === test_statuses_1.UPDATED) {
        return { expectedPath: exports.getReferencePath(formattedResult, stateName) };
    }
    if (status === test_statuses_1.FAIL) {
        return {
            actualPath: exports.getCurrentPath(formattedResult, stateName),
            diffPath: exports.getDiffPath(formattedResult, stateName),
            expectedPath: exports.getReferencePath(formattedResult, stateName),
        };
    }
    if (status === test_statuses_1.ERROR) {
        return {
            actualPath: formattedResult.state ? exports.getCurrentPath(formattedResult, stateName) : '',
        };
    }
    return {};
};
exports.hasImage = (formattedResult) => {
    return !!formattedResult.getImagesInfo().length
        || !!formattedResult.currentPath
        || !!formattedResult.screenshot;
};
exports.prepareCommonJSData = (data) => {
    return [
        `var data = ${JSON.stringify(data)};`,
        'try { module.exports = data; } catch(e) {}',
    ].join('\n');
};
